@@ -1,4 +1,5 @@
 import os
+import platform
 import shlex
 from collections.abc import Callable
 from concurrent.futures import ThreadPoolExecutor
@@ -8,10 +9,10 @@ from http import HTTPStatus
 from pathlib import Path
 from secrets import token_urlsafe
 from shutil import move
-from subprocess import run
+from subprocess import run  # nosec B404
 from tempfile import TemporaryDirectory, mkdtemp
 
-from urllib3.exceptions import HTTPError
+from urllib3.exceptions import HTTPError, ProtocolError, ReadTimeoutError
 from urllib3.poolmanager import PoolManager
 from urllib3.response import BaseHTTPResponse
 
@@ -23,10 +24,11 @@ from umu.umu_util import (
     extract_tarfile,
     file_digest,
     get_tempdir,
-    has_umu_setup,
+    has_runtime_installed,
     run_zenity,
     unix_flock,
     write_file_chunks,
+    write_install_marker,
 )
 
 RuntimeVersion = tuple[str, str, str]
@@ -75,11 +77,11 @@ def _install_umu(
     ret: int = 0  # Exit code from zenity
     thread_pool, http_pool = session_pools
     codename, variant, _ = runtime_ver
-    base_url: str = f"https://repo.steampowered.com/{variant}/images/latest-public-beta"
+    base_url: str = f"https://repo.steampowered.com/{variant.removesuffix('-arm64')}/images/latest-public-beta/"
     token: str = f"?versions={token_urlsafe(16)}"
     host: str = "repo.steampowered.com"
 
-    if codename.removeprefix("steamrt").isdigit():
+    if codename.removeprefix("steamrt").removesuffix("-arm64").isdigit():
         archive = f"SteamLinuxRuntime_{codename.removeprefix('steamrt')}.tar.xz"
     else:
         archive = f"SteamLinuxRuntime_{codename}.tar.xz"
@@ -109,7 +111,7 @@ def _install_umu(
     if not os.environ.get("UMU_ZENITY") or ret:
         digest: str = ""
         buildid: str = ""
-        endpoint: str = f"/{variant}/images/latest-public-beta"
+        endpoint: str = f"/{variant.removesuffix('-arm64')}/images/latest-public-beta"
         hashsum = sha256()
         headers: dict[str, str] | None = None
         cached_parts: Path
@@ -121,7 +123,7 @@ def _install_umu(
             preload_content=False,
         )
         if resp.status != HTTPStatus.OK:
-            err: str = f"{resp.getheader('Host')} returned the status: {resp.status}"
+            err: str = f"{host} returned the status: {resp.status}"
             raise HTTPError(err)
 
         # Parse data for the archive digest
@@ -139,7 +141,7 @@ def _install_umu(
             HTTPMethod.GET.value, f"{host}{endpoint}/BUILD_ID.txt{token}"
         )
         if resp.status != HTTPStatus.OK:
-            err: str = f"{resp.getheader('Host')} returned the status: {resp.status}"
+            err: str = f"{host} returned the status: {resp.status}"
             raise HTTPError(err)
 
         buildid = resp.data.decode(encoding="utf-8").strip()
@@ -174,14 +176,39 @@ def _install_umu(
             HTTPStatus.PARTIAL_CONTENT,
             HTTPStatus.REQUESTED_RANGE_NOT_SATISFIABLE,
         }:
-            err: str = f"{resp.getheader('Host')} returned the status: {resp.status}"
+            err: str = f"{host} returned the status: {resp.status}"
             raise HTTPError(err)
 
         # Download the runtime
         if resp.status != HTTPStatus.REQUESTED_RANGE_NOT_SATISFIABLE:
             try:
                 log.debug("Writing: %s", parts)
-                hashsum = write_file_chunks(parts, resp, hashsum)
+                attempts = 3
+                for trial in range(attempts):
+                    try:
+                        hashsum = write_file_chunks(parts, resp, hashsum)
+                    except (ProtocolError, ReadTimeoutError) as e:
+                        if trial == attempts - 1:
+                            raise e
+                        log.debug(e)
+                        log.info(
+                            "Connection broken, trying to resume %s (retry %s)",
+                            parts,
+                            trial + 1,
+                        )
+                        headers = {
+                            "Range": f"bytes={parts.stat().st_size}-",
+                        }
+                        resp = http_pool.request(
+                            HTTPMethod.GET.value,
+                            f"{host}{endpoint}/{archive}{token}",
+                            preload_content=False,
+                            headers=headers,
+                        )
+                        with parts.open("rb") as fp:
+                            hashsum = file_digest(fp, hashsum.name)
+                    else:
+                        break
             except HTTPError:
                 log.error("Aborting steamrt install due to network error")
                 log.info("Moving '%s' to cache for future resumption", parts.name)
@@ -225,7 +252,9 @@ def _install_umu(
 
         # Validate and post-install
         try:
-            check_runtime(local, runtime_ver)
+            ret = check_runtime(local, runtime_ver)
+            if not ret:
+                write_install_marker(local)
         finally:
             log.debug("Linking: umu -> _v2-entry-point")
             local.joinpath("umu").symlink_to("_v2-entry-point")
@@ -237,8 +266,14 @@ def setup_umu(
     """Install or update the runtime for the current user."""
     log.debug("Local: %s", local)
 
-    # New install or umu dir is empty
-    if not has_umu_setup(local):
+    # Backfill markers for installs created before markers existed after verifying.
+    if not has_runtime_installed(local) and local.is_dir():
+        ret = check_runtime(local, runtime_ver)
+        if not ret:
+            write_install_marker(local)
+
+    # New install
+    if not has_runtime_installed(local):
         log.debug("New install detected")
         log.info("Setting up Unified Launcher for Windows Games on Linux...")
         local.mkdir(parents=True, exist_ok=True)
@@ -246,7 +281,8 @@ def setup_umu(
             local,
             runtime_ver,
             session_pools,
-            lambda: local.joinpath("umu").is_file(),
+            # If marker is present, a successful install already occurred.
+            lambda: has_runtime_installed(local),
         )
         log.info("Using %s (latest)", runtime_ver[1])
         return
@@ -271,7 +307,7 @@ def _update_umu(
     resp: BaseHTTPResponse
     _, http_pool = session_pools
     codename, variant, _ = runtime_ver
-    endpoint: str = f"/{variant}/images/latest-public-beta"
+    endpoint: str = f"/{variant.removesuffix('-arm64')}/images/latest-public-beta"
     # Create a token and append it to the URL to avoid the Cloudflare cache
     # Avoids infinite updates to the runtime each launch
     # See https://github.com/Open-Wine-Components/umu-launcher/issues/188
@@ -283,17 +319,23 @@ def _update_umu(
 
     # Find the runtime directory (e.g., sniper_platform_0.20240530.90143)
     # Assume the directory begins with the variant
+    _codename = codename.removesuffix("-arm64")
     try:
-        max(file for file in local.glob(f"{codename}*") if file.is_dir())
+        max(file for file in local.glob(f"{_codename}_platform_*") if file.is_dir())
     except ValueError:
-        log.critical("*_platform_* directory missing in '%s'", local)
+        log.critical("%s_platform_* directory missing in '%s'", _codename, local)
         log.info("Restoring Runtime Platform...")
         _restore_umu(
             local,
             runtime_ver,
             session_pools,
-            lambda: len([file for file in local.glob(f"{codename}*") if file.is_dir()])
-            > 0,
+            lambda: bool(
+                [
+                    file
+                    for file in local.glob(f"{_codename}_platform_*")
+                    if file.is_dir()
+                ]
+            ),
         )
         return
 
@@ -326,7 +368,7 @@ def _update_umu(
     log.debug("Sending request to '%s' for 'VERSION.txt'...", url)
     resp = http_pool.request(HTTPMethod.GET.value, url)
     if resp.status != HTTPStatus.OK:
-        log.error("%s returned the status: %s", resp.getheader("Host"), resp.status)
+        log.error("%s returned the status: %s", host, resp.status)
         return
 
     # Update our runtime
@@ -348,11 +390,14 @@ def check_runtime(src: Path, runtime_ver: RuntimeVersion) -> int:
     ret: int = 1
 
     # Find the runtime directory
+    _codename = codename.removesuffix("-arm64")
     try:
-        runtime = max(file for file in src.glob(f"{codename}*") if file.is_dir())
+        runtime = max(
+            file for file in src.glob(f"{_codename}_platform_*") if file.is_dir()
+        )
     except ValueError:
         log.critical("%s validation failed", variant)
-        log.critical("Could not find *_platform_* in '%s'", src)
+        log.critical("Could not find %s_platform_* in '%s'", _codename, src)
         return ret
 
     if not pv_verify.is_file():
@@ -361,14 +406,9 @@ def check_runtime(src: Path, runtime_ver: RuntimeVersion) -> int:
         return ret
 
     log.info("Verifying integrity of %s...", runtime.name)
-    ret = run(
-        [
-            pv_verify,
-            "--quiet",
-            "--minimized-runtime",
-            runtime.joinpath("files"),
-        ],
-        check=False,
+    pv = Path(pv_verify).expanduser().resolve(strict=True)
+    ret = run(  # nosec B603
+        [str(pv), "--quiet", "--minimized-runtime", str(runtime / "files")], check=False
     ).returncode
 
     if ret:
@@ -431,6 +471,7 @@ class UmuRuntime:
     name: str
     variant: str
     appid: str
+    machine: str
     path: Path | None = None
 
     def __post_init__(self) -> None:  # noqa: D105
@@ -441,7 +482,11 @@ class UmuRuntime:
 
     def __bool__(self) -> bool:
         """Return if the runtime's path has been populated."""
-        return self.path is not None and self.path.is_dir() and self.path.joinpath("mtree.txt.gz").is_file()
+        return (
+            self.path is not None
+            and self.path.is_dir()
+            and self.path.joinpath("mtree.txt.gz").is_file()
+        )
 
     def as_tuple(self) -> RuntimeVersion:
         """Return runtime information as tuple."""
@@ -449,14 +494,27 @@ class UmuRuntime:
 
 
 RUNTIME_VERSIONS = {
-    "host": UmuRuntime("host", "", ""),
-    "1391110": UmuRuntime("soldier",      "steamrt2", "1391110"),
-    "1628350": UmuRuntime("sniper",       "steamrt3", "1628350"),
-    "3810310": UmuRuntime("sniper-arm64", "steamrt3", "3810310"),
-    "4183110": UmuRuntime("steamrt4",     "steamrt4", "4183110"),
-    # "4183110": UmuRuntime("steamrt4-arm64", "steamrt4", "4183110"),
+    "host": UmuRuntime("host", "", "", platform.machine()),
 }
 
+# fmt: off
+RUNTIME_VERSIONS.update({
+    "1391110": UmuRuntime("soldier",        "steamrt2",       "1391110", "x86_64"),
+    "1628350": UmuRuntime("sniper",         "steamrt3",       "1628350", "x86_64"),
+    "3810310": UmuRuntime("sniper-arm64",   "steamrt3-arm64", "3810310", "aarch64"),
+    "4183110": UmuRuntime("steamrt4",       "steamrt4",       "4183110", "x86_64"),
+    "4185400": UmuRuntime("steamrt4-arm64", "steamrt4-arm64", "4185400", "aarch64"),
+})
+# fmt: on
+
+if platform.machine() == "x86_64":  # noqa: SIM114
+    pass
+elif platform.machine() == "aarch64":
+    pass
+else:
+    err: str = f"Unsupported platform {platform.machine()}"
+    raise RuntimeError(err)
+
 
 RUNTIME_NAMES = {RUNTIME_VERSIONS[key].name: key for key in RUNTIME_VERSIONS}
 
@@ -479,7 +537,9 @@ class CompatLayer:
             with path.joinpath("compatibilitytool.vdf").open(encoding="utf-8") as f:
                 # There can be multiple tools definitions in `compatibilitytools.vdf`
                 # Take the first one and hope it is the one with the correct display_name
-                compat_tools = tuple(vdf.load(f)["compatibilitytools"]["compat_tools"].values())
+                compat_tools = tuple(
+                    vdf.load(f)["compatibilitytools"]["compat_tools"].values()
+                )
                 self.compatibility_tool = compat_tools[0]
         else:
             self.compatibility_tool = {"display_name": path.name}
@@ -489,7 +549,10 @@ class CompatLayer:
 
     def _resolve(self, shim: Path) -> "CompatLayer | None":
         """Construct and provide the concrete CompatLayer this layer depends on."""
-        if self.required_tool_appid is not None and self.required_runtime.path is not None:
+        if (
+            self.required_tool_appid is not None
+            and self.required_runtime.path is not None
+        ):
             return CompatLayer(self.required_runtime.path, shim)
         return None
 
@@ -503,7 +566,9 @@ class CompatLayer:
     @property
     def required_tool_appid(self) -> str | None:
         """Report the appid of the tool this CompatLayer requires."""
-        return str(ret) if (ret := self.tool_manifest.get("require_tool_appid")) else None
+        return (
+            str(ret) if (ret := self.tool_manifest.get("require_tool_appid")) else None
+        )
 
     @property
     def required_runtime(self) -> UmuRuntime:
@@ -514,7 +579,11 @@ class CompatLayer:
 
     @property
     def layer_name(self) -> str:  # noqa: D102
-        return str(ret) if (ret := self.tool_manifest.get("compatmanager_layer_name")) else ""
+        return (
+            str(ret)
+            if (ret := self.tool_manifest.get("compatmanager_layer_name"))
+            else ""
+        )
 
     @property
     def launcher_service(self) -> str:
@@ -541,7 +610,9 @@ class CompatLayer:
     @property
     def display_name(self) -> str | None:
         """Report the name of this CompatLayer as set in its manifest."""
-        return str(ret) if (ret := self.compatibility_tool.get("display_name")) else None
+        return (
+            str(ret) if (ret := self.compatibility_tool.get("display_name")) else None
+        )
 
     @property
     def has_runtime(self) -> bool:
@@ -552,9 +623,6 @@ class CompatLayer:
         """Return the tool specific entry point."""
         tool_path = os.path.normpath(self.tool_path)
         cmd = "".join([shlex.quote(tool_path), self.tool_manifest["commandline"]])
-        # Temporary override entry point for backwards compatibility
-        if self.layer_name in {"container-runtime"}:
-            cmd = cmd.replace("_v2-entry-point", "umu")
         cmd = cmd.replace("%verb%", verb)
         return shlex.split(cmd)
 
@@ -563,10 +631,18 @@ class CompatLayer:
 
         If the runtime uses another runtime, its entry point is prepended to the local command.
         """
-        log.info("Running '%s' using runtime '%s'", self.display_name, self.required_runtime.name)
-        cmd = self.runtime.command(verb, unwrapped=False) if self.runtime is not None else []
+        log.info(
+            "Running '%s' using runtime '%s'",
+            self.display_name,
+            self.required_runtime.name,
+        )
+        cmd = (
+            self.runtime.command(verb, unwrapped=False)
+            if self.runtime is not None
+            else []
+        )
         target = self._unwrapped_cmd(verb)
-        if self.layer_name in {"container-runtime"}:
+        if self.layer_name == "container-runtime":
             cmd.extend([*target, self._shim.as_posix()])
         elif self.runtime is None:
             cmd.extend([self._shim.as_posix(), *target])
@@ -574,7 +650,7 @@ class CompatLayer:
             cmd.extend(target)
         return cmd
 
-    def command(self, verb:str, *, unwrapped: bool) -> list[str]:
+    def command(self, verb: str, *, unwrapped: bool) -> list[str]:
         """Return the tool's fully qualified (wrapped) or tool specific (unwrapped) entry point."""
         if unwrapped:
             return self._unwrapped_cmd(verb)
@@ -582,4 +658,3 @@ class CompatLayer:
 
     def as_str(self, verb: str):  # noqa: D102
         return " ".join(map(shlex.quote, self.command(verb, unwrapped=False)))
-
